@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2003 Nominum, Inc.
+# Copyright (C) 2001-2004 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -13,7 +13,7 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 # OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: rdata.py,v 1.20 2003/06/30 06:08:27 halley Exp $
+# $Id: rdata.py,v 1.26 2004/03/19 00:17:27 halley Exp $
 
 """DNS rdata.
 
@@ -38,6 +38,15 @@ import dns.tokenizer
 _hex_chunksize = 32
 
 def _hexify(data, chunksize=None):
+    """Convert a binary string into its hex encoding, broken up into chunks
+    of I{chunksize} characters separated by a space.
+
+    @param data: the binary string
+    @type data: string
+    @param chunksize: the chunk size.  Default is L{dns.rdata._hex_chunksize}
+    @rtype: string
+    """
+    
     if chunksize is None:
         chunksize = _hex_chunksize
     hex = data.encode('hex_codec')
@@ -54,6 +63,16 @@ def _hexify(data, chunksize=None):
 _base64_chunksize = 32
 
 def _base64ify(data, chunksize=None):
+    """Convert a binary string into its base64 encoding, broken up into chunks
+    of I{chunksize} characters separated by a space.
+
+    @param data: the binary string
+    @type data: string
+    @param chunksize: the chunk size.  Default is
+    L{dns.rdata._base64_chunksize}
+    @rtype: string
+    """
+
     if chunksize is None:
         chunksize = _base64_chunksize
     b64 = data.encode('base64_codec')
@@ -73,12 +92,17 @@ __escaped = {
     '\\' : True,
     }
 
-def _escapify(label):
+def _escapify(qstring):
     """Escape the characters in a quoted string which need it.
+
+    @param qstring: the string
+    @type qstring: string
     @returns: the escaped string
-    @rtype: string"""
+    @rtype: string
+    """
+    
     text = ''
-    for c in label:
+    for c in qstring:
         if c in __escaped:
             text += '\\' + c
         elif ord(c) >= 0x20 and ord(c) < 0x7F:
@@ -89,7 +113,12 @@ def _escapify(label):
 
 def _truncate_bitmap(what):
     """Determine the index of greatest byte that isn't all zeros, and
-    return the bitmap that contains all the bytes less than that index."""
+    return the bitmap that contains all the bytes less than that index.
+
+    @param what: a string of octets representing a bitmap.
+    @type what: string
+    @rtype: string
+    """
     
     for i in xrange(len(what) - 1, -1, -1):
         if what[i] != '\x00':
@@ -99,24 +128,28 @@ def _truncate_bitmap(what):
 class Rdata(object):
     """Base class for all DNS rdata types.
     """
+
+    __slots__ = ['rdclass', 'rdtype']
     
     def __init__(self, rdclass, rdtype):
         """Initialize an rdata.
         @param rdclass: The rdata class
         @type rdclass: int
         @param rdtype: The rdata type
-        @type rdtype: int"""
+        @type rdtype: int
+        """
 
 	self.rdclass = rdclass
 	self.rdtype = rdtype
 
     def covers(self):
-        """DNS SIG rdatas apply to a specific type; this type is
+        """DNS SIG/RRSIG rdatas apply to a specific type; this type is
         returned by the covers() function.  If the rdata type is not
-        SIG, dns.rdatatype.NONE is returned.  This is useful when
-        creating rdatasets, allowing the rdataset to contain only SIGs
-        of a particular type, e.g. SIG(NS).
-        @rtype: int"""
+        SIG or RRSIG, dns.rdatatype.NONE is returned.  This is useful when
+        creating rdatasets, allowing the rdataset to contain only RRSIGs
+        of a particular type, e.g. RRSIG(NS).
+        @rtype: int
+        """
         
 	return dns.rdatatype.NONE
 
@@ -124,17 +157,22 @@ class Rdata(object):
         """Return a 32-bit type value, the least significant 16 bits of
         which are the ordinary DNS type, and the upper 16 bits of which are
         the "covered" type, if any.
-        @rtype: int"""
+        @rtype: int
+        """
+        
         return self.covers() << 16 | self.rdtype
 
     def to_text(self, origin=None, relativize=True, **kw):
         """Convert an rdata to text format.
-        @rtype: string"""
+        @rtype: string
+        """
         raise NotImplementedError        
 
     def to_wire(self, file, compress = None, origin = None):
         """Convert an rdata to wire format.
-        @rtype: string"""
+        @rtype: string
+        """
+        
         raise NotImplementedError
         
     def __repr__(self):
@@ -155,6 +193,7 @@ class Rdata(object):
         rdclass.  Return < 0 if self < other in the DNSSEC ordering,
         0 if self == other, and > 0 if self > other.
         """
+        
         raise NotImplementedError
     
     def __eq__(self, other):
@@ -214,7 +253,8 @@ class Rdata(object):
         @type origin: dns.name.Name
         @param relativize: should names be relativized?
         @type origin: bool
-        @rtype: dns.rdata.Rdata instance"""
+        @rtype: dns.rdata.Rdata instance
+        """
 
         raise NotImplementedError
 
@@ -235,18 +275,29 @@ class Rdata(object):
         @type rdlen: int
         @param origin: The origin to use for relative names
         @type origin: dns.name.Name
-        @rtype: dns.rdata.Rdata instance"""
+        @rtype: dns.rdata.Rdata instance
+        """
 
         raise NotImplementedError
 
     from_wire = classmethod(from_wire)
-    
-     
+
+    def choose_relativity(self, origin = None, relativize = True):
+        """Convert any domain names in the rdata to the specified
+        relativization.
+        """
+        
+        pass
+
+
 class GenericRdata(Rdata):
     """Generate Rdata Class
 
     This class is used for rdata types for which we have no better
-    implementation.  It implements the DNS "unknown RRs" scheme."""
+    implementation.  It implements the DNS "unknown RRs" scheme.
+    """
+
+    __slots__ = ['data']
     
     def __init__(self, rdclass, rdtype, data):
         super(GenericRdata, self).__init__(rdclass, rdtype)
