@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2003 Nominum, Inc.
+# Copyright (C) 2001-2004 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -13,7 +13,7 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 # OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: rdatatype.py,v 1.12 2003/06/24 02:37:00 halley Exp $
+# $Id: rdatatype.py,v 1.15 2004/03/19 00:17:27 halley Exp $
 
 """DNS Rdata Types.
 
@@ -21,12 +21,12 @@
 @type _by_text: dict
 @var _by_value: The rdata type value to textual name mapping
 @type _by_value: dict
-@var metatypes: If an rdatatype is a metatype, there will be a mapping
+@var _metatypes: If an rdatatype is a metatype, there will be a mapping
 whose key is the rdatatype value and whose value is True in this dictionary.
-@type metatypes: dict
-@var singletons: If an rdatatype is a singleton, there will be a mapping
+@type _metatypes: dict
+@var _singletons: If an rdatatype is a singleton, there will be a mapping
 whose key is the rdatatype value and whose value is True in this dictionary.
-@type singletons: dict"""
+@type _singletons: dict"""
 
 import re
 
@@ -72,6 +72,9 @@ DNAME = 39
 OPT = 41
 APL = 42
 DS = 43
+RRSIG = 46
+NSEC = 47
+DNSKEY = 48
 UNSPEC = 103
 TKEY = 249
 TSIG = 250
@@ -122,6 +125,9 @@ _by_text = {
     'OPT' : OPT,
     'APL' : APL,
     'DS' : DS,
+    'RRSIG' : RRSIG,
+    'NSEC' : NSEC,
+    'DNSKEY' : DNSKEY,
     'UNSPEC' : UNSPEC,
     'TKEY' : TKEY,
     'TSIG' : TSIG,
@@ -132,62 +138,18 @@ _by_text = {
     'ANY' : ANY
     }
 
-_by_value = {
-    NONE : 'NONE',
-    A : 'A',
-    NS : 'NS',
-    MD : 'MD',
-    MF : 'MF',
-    CNAME : 'CNAME',
-    SOA : 'SOA',
-    MB : 'MB',
-    MG : 'MG',
-    MR : 'MR',
-    NULL : 'NULL',
-    WKS : 'WKS',
-    PTR : 'PTR',
-    HINFO : 'HINFO',
-    MINFO : 'MINFO',
-    MX : 'MX',
-    TXT : 'TXT',
-    RP : 'RP',
-    AFSDB : 'AFSDB',
-    X25 : 'X25',
-    ISDN : 'ISDN',
-    RT : 'RT',
-    NSAP : 'NSAP',
-    NSAP_PTR : 'NSAP-PTR',
-    SIG : 'SIG',
-    KEY : 'KEY',
-    PX : 'PX',
-    GPOS : 'GPOS',
-    AAAA : 'AAAA',
-    LOC : 'LOC',
-    NXT : 'NXT',
-    SRV : 'SRV',
-    NAPTR : 'NAPTR',
-    KX : 'KX',
-    CERT : 'CERT',
-    A6 : 'A6',
-    DNAME : 'DNAME',
-    OPT : 'OPT',
-    APL : 'APL',
-    DS : 'DS',
-    UNSPEC : 'UNSPEC',
-    TKEY : 'TKEY',
-    TSIG : 'TSIG',
-    IXFR : 'IXFR',
-    AXFR : 'AXFR',
-    MAILB : 'MAILB',
-    MAILA : 'MAILA',
-    ANY : 'ANY'
-    }
+# We construct the inverse mapping programmatically to ensure that we
+# cannot make any mistakes (e.g. omissions, cut-and-paste errors) that
+# would cause the mapping not to be true inverse.
 
-metatypes = {
+_by_value = dict([(y, x) for x, y in _by_text.iteritems()])
+
+
+_metatypes = {
     OPT : True
     }
 
-singletons = {
+_singletons = {
     SOA : True,
     NXT : True,
     DNAME : True,
@@ -202,6 +164,10 @@ class UnknownRdatatype(dns.exception.DNSException):
 
 def from_text(text):
     """Convert text into a DNS rdata type value.
+    @param text: the text
+    @type text: string
+    @raises dns.rdatatype.UnknownRdatatype: the type is unknown
+    @raises ValueError: the rdata type value is not >= 0 and <= 65535
     @rtype: int"""
     
     value = _by_text.get(text.upper())
@@ -210,14 +176,19 @@ def from_text(text):
         if match == None:
             raise UnknownRdatatype
         value = int(match.group(1))
+        if value < 0 or value > 65535:
+            raise ValueError, "type must be between >= 0 and <= 65535"
     return value
 
 def to_text(value):
     """Convert a DNS rdata type to text.
+    @param value: the rdata type value
+    @type value: int
+    @raises ValueError: the rdata type value is not >= 0 and <= 65535
     @rtype: string"""
     
     if value < 0 or value > 65535:
-        raise UnknownRdatatype
+        raise ValueError, "type must be between >= 0 and <= 65535"
     text = _by_value.get(value)
     if text is None:
         text = 'TYPE' + `value`
@@ -225,16 +196,20 @@ def to_text(value):
 
 def is_metatype(rdtype):
     """True if the type is a metatype.
+    @param rdtype: the type
+    @type rdtype: int
     @rtype: bool"""
     
-    if type >= TKEY and type <= ANY or metatypes.has_key(rdtype):
+    if rdtype >= TKEY and rdtype <= ANY or _metatypes.has_key(rdtype):
         return True
     return False
 
 def is_singleton(rdtype):
     """True if the type is a singleton.
+    @param rdtype: the type
+    @type rdtype: int
     @rtype: bool"""
     
-    if singletons.has_key(rdtype):
+    if _singletons.has_key(rdtype):
         return True
     return False
