@@ -1,4 +1,4 @@
-# Copyright (C) 2003 Nominum, Inc.
+# Copyright (C) 2003, 2004 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -13,7 +13,7 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 # OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: resolver.py,v 1.20 2003/10/27 00:48:49 halley Exp $
+# $Id: resolver.py,v 1.22 2004/03/19 00:17:27 halley Exp $
 
 """DNS stub resolver.
 
@@ -164,9 +164,12 @@ class Cache(object):
         
         now = time.time()
         if self.next_cleaning <= now:
+            keys_to_delete = []
             for (k, v) in self.data.iteritems():
                 if v.expiration <= now:
-                    del self.data[k]
+                    keys_to_delete.append(k)
+            for k in keys_to_delete:
+                del self.data[k]
             now = time.time()
             self.next_cleaning = now + self.cleaning_interval
             
